@@ -37,10 +37,10 @@ KDE_NO_CDTOR_EXPORT FileDocument::FileDocument (short i, const String &s, KMPlay
     id = i;
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr FileDocument::childFromTag(const String &tag) {
+KDE_NO_EXPORT KMPlayer::Node *FileDocument::childFromTag(const String &tag) {
     if (tag == String (nodeName ()))
         return this;
-    return 0L;
+    return NULL;
 }
 
 void FileDocument::readFromFile (const String & fn) {
@@ -84,7 +84,7 @@ KDE_NO_EXPORT void Recents::defer () {
     }
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr Recents::childFromTag (const String & tag) {
+KDE_NO_EXPORT KMPlayer::Node *Recents::childFromTag (const String & tag) {
     // kdDebug () << nodeName () << " childFromTag " << tag << endl;
     if (tag == String ("item"))
         return new Recent (m_doc, app);
@@ -125,12 +125,12 @@ Group::Group (KMPlayer::NodePtr & doc, Application * a, const String & pn)
         setAttribute (KMPlayer::StringPool::attr_title, pn);
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr Group::childFromTag (const String & tag) {
+KDE_NO_EXPORT KMPlayer::Node *Group::childFromTag (const String & tag) {
     if (tag == String ("item"))
         return new Recent (m_doc, app);
     else if (tag == String ("group"))
         return new Group (m_doc, app);
-    return 0L;
+    return NULL;
 }
 
 KDE_NO_EXPORT void Group::closed () {
@@ -168,7 +168,7 @@ KDE_NO_CDTOR_EXPORT Playlist::Playlist (Application *a, KMPlayer::PlayListNotify
     pretty_name = "Persistent Playlists";
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr Playlist::childFromTag (const String & tag) {
+KDE_NO_EXPORT KMPlayer::Node *Playlist::childFromTag (const String & tag) {
     // kdDebug () << nodeName () << " childFromTag " << tag << endl;
     const char * name = (const char *) tag;
     if (!strcmp (name, "item"))
@@ -269,7 +269,7 @@ PlaylistGroup::PlaylistGroup (KMPlayer::NodePtr & doc, Application * a, bool lm)
   : KMPlayer::Mrl (doc, KMPlayer::id_node_group_node), app (a), playmode (lm) {
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr PlaylistGroup::childFromTag (const String & tag) {
+KDE_NO_EXPORT KMPlayer::Node *PlaylistGroup::childFromTag (const String & tag) {
     const char * name = (const char *) tag;
     if (!strcmp (name, "item"))
         return new PlaylistItem (m_doc, app, playmode);
@@ -277,7 +277,7 @@ KDE_NO_EXPORT KMPlayer::NodePtr PlaylistGroup::childFromTag (const String & tag)
         return new PlaylistGroup (m_doc, app, playmode);
     else if (!strcmp (name, "object"))
         return new HtmlObject (m_doc, app, playmode);
-    return 0L;
+    return NULL;
 }
 
 KDE_NO_EXPORT void PlaylistGroup::closed () {
@@ -323,7 +323,7 @@ KDE_NO_EXPORT void HtmlObject::closed () {
     PlaylistItemBase::closed ();
 }
 
-KDE_NO_EXPORT KMPlayer::NodePtr HtmlObject::childFromTag (const String & tag) {
+KDE_NO_EXPORT KMPlayer::Node *HtmlObject::childFromTag (const String & tag) {
     const char * name = (const char *) tag;
     if (!strcasecmp (name, "param"))
         return new DarkNode (m_doc, name, KMPlayer::id_node_param);
