@@ -37,7 +37,7 @@ static const short id_node_html_embed = 34;
 class KMPLAYER_NO_EXPORT FileDocument : public KMPlayer::Document {
 public:
     FileDocument (short id, const String &, KMPlayer::PlayListNotify * notify = 0L);
-    KMPlayer::NodePtr childFromTag (const String & tag);
+    KMPlayer::Node *childFromTag (const String & tag);
     void readFromFile (const String & file);
     void writeToFile (const String & file);
     unsigned int load_tree_version;
@@ -60,7 +60,7 @@ public:
     void defer ();
     void activate ();
     void *message (KMPlayer::MessageType msg, void *content=NULL);
-    KMPlayer::NodePtr childFromTag (const String & tag);
+    KMPlayer::Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "playlist"; }
     Application * app;
 };
@@ -77,7 +77,7 @@ public:
 class KMPLAYER_NO_EXPORT Group : public KMPlayer::Mrl {
 public:
     Group (KMPlayer::NodePtr &doc, Application *a, const String &pn=String());
-    KMPlayer::NodePtr childFromTag (const String & tag);
+    KMPlayer::Node *childFromTag (const String & tag);
     void defer () {} // TODO lazy loading of largish sub trees
     void closed ();
     KDE_NO_EXPORT const char * nodeName () const { return "group"; }
@@ -90,7 +90,7 @@ public:
     void *message (KMPlayer::MessageType msg, void *content=NULL);
     void defer ();
     void activate ();
-    KMPlayer::NodePtr childFromTag (const String & tag);
+    KMPlayer::Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "playlist"; }
     Application * app;
     bool playmode;
@@ -118,7 +118,7 @@ class KMPLAYER_NO_EXPORT PlaylistGroup : public KMPlayer::Mrl {
 public:
     PlaylistGroup (KMPlayer::NodePtr &doc, Application *a, const String &pn);
     PlaylistGroup (KMPlayer::NodePtr &doc, Application *a, bool plmode=false);
-    KMPlayer::NodePtr childFromTag (const String & tag);
+    KMPlayer::Node *childFromTag (const String & tag);
     void closed ();
     void setNodeName (const String &);
     KDE_NO_EXPORT const char * nodeName () const { return "group"; }
@@ -131,7 +131,7 @@ public:
     HtmlObject (KMPlayer::NodePtr & doc, Application *a, bool playmode);
     void activate ();
     void closed ();
-    KMPlayer::NodePtr childFromTag (const String & tag);
+    KMPlayer::Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "object"; }
 };
 
