@@ -22,19 +22,19 @@
 
 using namespace KMPlayer;
 
-KMPLAYER_NO_EXPORT NodePtr RSS::Rss::childFromTag (const String & tag) {
+KMPLAYER_NO_EXPORT Node *RSS::Rss::childFromTag (const String & tag) {
     if (!strcmp ((const char *) tag, "channel"))
         return new RSS::Channel (m_doc);
-    return 0L;
+    return NULL;
 }
 
-KMPLAYER_NO_EXPORT NodePtr RSS::Channel::childFromTag (const String & tag) {
+KMPLAYER_NO_EXPORT Node *RSS::Channel::childFromTag (const String & tag) {
     const char *ctag = (const char *) tag;
     if (!strcmp (ctag, "item"))
         return new RSS::Item (m_doc);
     else if (!strcmp (ctag, "title"))
         return new DarkNode (m_doc, tag, id_node_title);
-    return 0L;
+    return NULL;
 }
 
 KMPLAYER_NO_EXPORT void RSS::Channel::closed () {
@@ -50,7 +50,7 @@ KMPLAYER_NO_EXPORT bool RSS::Channel::expose () const {
         previousSibling () || nextSibling ();
 }
 
-KMPLAYER_NO_EXPORT NodePtr RSS::Item::childFromTag (const String & tag) {
+KMPLAYER_NO_EXPORT Node *RSS::Item::childFromTag (const String & tag) {
     const char *ctag = (const char *) tag;
     if (!strcmp (ctag, "enclosure"))
         return new RSS::Enclosure (m_doc);
@@ -58,7 +58,7 @@ KMPLAYER_NO_EXPORT NodePtr RSS::Item::childFromTag (const String & tag) {
         return new DarkNode (m_doc, tag, id_node_title);
     else if (!strcmp (ctag, "description"))
         return new DarkNode (m_doc, tag, id_node_description);
-    return 0L;
+    return NULL;
 }
 
 KMPLAYER_NO_EXPORT void RSS::Item::closed () {
