@@ -162,6 +162,7 @@ private:
     void setDurationItem (DurationTime item, const String & val);
     void propagateStop (bool forced);
     void propagateStart ();
+    int repeat;
 };
 
 class KMPLAYER_NO_EXPORT MouseListeners {
@@ -201,12 +202,17 @@ const short id_node_audio_video = 121;
 const short id_node_text = 122;
 const short id_node_ref = 123;
 const short id_node_brush = 124;
+const short id_node_smil_text = 125;
+const short id_node_text_styling = 126;
 const short id_node_set = 132;
 const short id_node_animate = 133;
 const short id_node_title = 140;
 const short id_node_param = 141;
 const short id_node_meta = 142;
 const short id_node_priorityclass = 143;
+const short id_node_div = 144;
+const short id_node_span = 145;
+const short id_node_p = 146;
 const short id_node_anchor = 150;
 const short id_node_area = 151;
 const short id_node_first = id_node_smil;
@@ -225,7 +231,7 @@ const short id_node_last = 200; // reserve 100 ids
 class Smil : public Mrl {
 public:
     KDE_NO_CDTOR_EXPORT Smil (NodePtr & d) : Mrl (d, id_node_smil) {}
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "smil"; }
     PlayType playType () { return play_type_video; }
     void activate ();
@@ -247,7 +253,7 @@ public:
 class KMPLAYER_NO_EXPORT Head : public Element {
 public:
     KDE_NO_CDTOR_EXPORT Head (NodePtr & d) : Element (d, id_node_head) {}
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "head"; }
     void closed ();
     void *message (MessageType msg, void *content=NULL);
@@ -260,7 +266,7 @@ public:
 class KMPLAYER_NO_EXPORT Layout : public Element {
 public:
     Layout (NodePtr & d);
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return "layout"; }
     void closed ();
     void *message (MessageType msg, void *content=NULL);
@@ -338,7 +344,7 @@ class KMPLAYER_NO_EXPORT Region : public RegionBase {
 public:
     Region (NodePtr & d);
     KDE_NO_EXPORT const char * nodeName () const { return "region"; }
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     void calculateBounds (Single w, Single h);
     void *message (MessageType msg, void *content=NULL);
 private:
@@ -414,7 +420,7 @@ public:
 class KMPLAYER_NO_EXPORT GroupBase : public Element {
 public:
     ~GroupBase ();
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     PlayType playType () { return play_type_none; }
     void parseParam (const TrieString &name, const String &value);
     void init ();
@@ -477,7 +483,7 @@ class KMPLAYER_NO_EXPORT Excl : public GroupBase {
 public:
     KDE_NO_CDTOR_EXPORT Excl (NodePtr & d) : GroupBase (d, id_node_excl) {}
     KDE_NO_EXPORT const char * nodeName () const { return "excl"; }
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     void begin ();
     void deactivate ();
     void *message (MessageType msg, void *content=NULL);
@@ -498,7 +504,7 @@ public:
     KDE_NO_CDTOR_EXPORT PriorityClass (NodePtr &d)
         : Element (d, id_node_priorityclass) {}
     KDE_NO_EXPORT const char * nodeName () const { return "priorityClass"; }
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     void init ();
     void parseParam (const TrieString &, const String &);
     void *message (MessageType msg, void *content=NULL);
@@ -547,7 +553,7 @@ public:
     void activate ();
     void *message (MessageType msg, void *content=NULL);
     KDE_NO_EXPORT const char * nodeName () const { return "a"; }
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     KDE_NO_EXPORT void accept (Visitor * v) { v->visit (this); }
 };
 
@@ -574,7 +580,7 @@ public:
     MediaType (NodePtr & d, const String & t, short id);
     ~MediaType ();
 
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     KDE_NO_EXPORT const char * nodeName () const { return (const char*)m_type; }
     virtual void closed ();
     virtual void init ();
@@ -634,11 +640,10 @@ protected:
 class KMPLAYER_NO_EXPORT AVMediaType : public MediaType {
 public:
     AVMediaType (NodePtr & d, const String & t);
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     virtual void begin ();
     virtual void undefer ();
     virtual void finish ();
-    virtual void endOfFile ();
     virtual void accept (Visitor *);
     virtual bool expose () const;
     virtual void parseParam (const TrieString &, const String &);
@@ -648,7 +653,7 @@ public:
 class KMPLAYER_NO_EXPORT ImageMediaType : public MediaType {
 public:
     ImageMediaType (NodePtr & d);
-    NodePtr childFromTag (const String & tag);
+    Node *childFromTag (const String & tag);
     PlayType playType () { return play_type_image; }
     virtual void activate ();
     virtual void begin ();
@@ -689,6 +694,64 @@ public:
     virtual void accept (Visitor *);
 };
 
+class KMPLAYER_NO_EXPORT SmilText : public Element {
+public:
+    SmilText (NodePtr &doc);
+    ~SmilText ();
+    virtual void init ();
+    virtual void activate ();
+    virtual void begin ();
+    virtual void finish ();
+    virtual void deactivate ();
+    virtual void reset ();
+    virtual bool expose () const { return false; }
+    KDE_NO_EXPORT const char *nodeName () const { return "smilText"; }
+    Node *childFromTag (const String & tag);
+    virtual void parseParam (const TrieString &name, const String &value);
+    virtual void *message (MessageType msg, void *content=NULL);
+    virtual void accept (Visitor *v) { v->visit (this); }
+
+    String richText ();
+
+    SurfacePtrW text_surface;
+    NodePtrW region_node;
+    ConnectionPtr region_attach;
+    Runtime *runtime;
+    bool inited;
+};
+
+class KMPLAYER_NO_EXPORT TextFlow : public Element {
+public:
+    TextFlow (NodePtr &doc, short id, const String &tag);
+    ~TextFlow ();
+
+    virtual void init ();
+    virtual void activate ();
+    virtual bool expose () const { return false; }
+    KDE_NO_EXPORT const char *nodeName () const { return (const char *) tag; }
+    Node *childFromTag (const String &tag);
+    virtual void parseParam (const TrieString &name, const String &value);
+    virtual void accept (Visitor *v) { v->visit (this); }
+
+    int font_color;
+    int background_color;
+    enum { DirLtr, DirRtl, DirLtro, DirRtlo, DirInherit } text_direction;
+    String font_family;
+    int font_size;
+    enum {
+        StyleNormal, StyleItalic, StyleOblique, StyleRevOblique, StyleInherit
+    } font_style;
+    enum { WeightNormal, WeightBold, WeightInherit } font_weight;
+    enum { ModeAppend, ModeReplace, ModeInherit } text_mode;
+    enum { PlaceStart, PlaceCenter, PlaceEnd, PlaceInherit } text_place;
+    String text_style;
+    enum { Wrap, NoWrap, WrapInherit } text_wrap;
+    enum { SpaceDefault, SpacePreserve } space;
+    enum { WritingLrTb, WritingRlTb, WritingTbLr, WritingTbRl } text_writing;
+
+    String tag;
+};
+
 class KMPLAYER_NO_EXPORT AnimateGroup : public Element {
 public:
     ~AnimateGroup ();
