@@ -328,7 +328,7 @@ void Control::playCurrent () {
             updateViewAreaSizes ();
             if (m_process->hasXv ())
                 XSync (GDK_DISPLAY_XDISPLAY (gtk_widget_get_display (VIEWAREA(m_app)->viewWidget ())), false);
-            VIEWAREA(m_app)->setEventFiltering (true);
+            VIEWAREA(m_app)->setEventFiltering (m_app, true);
             m_process->play (mrl);
         } else
             errorLog() << "playCurrent and mrl not resolved" << endl;
@@ -566,15 +566,34 @@ void Control::readPlaylist (NodePtr root, TextStream & in) {
                 aspect = height > 0 ? width/height : 0.0;
             }
         }
-    } else if (line.lower () != String ("[reference]")) do {
-        if (line.lower ().startsWith (String ("asf ")))
-            line = line.mid (4).stripWhiteSpace ();
-        if (!line.isEmpty () && !line.startsWith (Char ('#')))
-            cur_elm->appendChild(new GenericURL(playlist_data->document, line));
-        line.truncate (0);
-        while (!in.atEnd () && line.isEmpty ())
+    } else if (line.lower () != String ("[reference]")) {
+        bool extm3u = line.startsWith ("#EXTM3U");
+        String title;
+        if (extm3u)
             line = in.readLine ().stripWhiteSpace ();
-    } while (!line.isEmpty () && line != String ("--stop--")); /* TODO && m_document.size () < 1024 / * support 1k entries * /);*/
+        while (!line.isNull ()) {
+            if (!line.isEmpty ()) {
+                if (line == String ("--stop--"))
+                    break;
+                if (line.lower ().startsWith (String ("asf ")))
+                    line = line.mid (4).stripWhiteSpace ();
+                if (!line.isEmpty ()) {
+                    if (extm3u && line.startsWith ("#")) {
+                        if (line.startsWith ("#EXTINF:"))
+                            title = line.mid (9);
+                        else
+                            title = line.mid (1);
+                    } else if (!line.startsWith (Char ('#'))) {
+                        cur_elm->appendChild (
+                                new GenericURL (playlist_data->document,
+                                    line, title));
+                        title.truncate (0);
+                    }
+                }
+                line = in.readLine ().stripWhiteSpace ();
+            }
+        }
+    }
 }
 
 ActorAgent *Control::actorAgent () const {
@@ -823,7 +842,7 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
     if (!p->mrl () && ns > Process::Ready) {
         p->stop (); // reschedule to the NotRunning state
     } else if (ns == Process::NotRunning) {
-        VIEWAREA(m_app)->setEventFiltering (false);
+        VIEWAREA(m_app)->setEventFiltering (m_app, false);
         if (p == m_process) {
             if (playlist_data->document &&
                     playlist_data->document->state > Node::state_init) {
@@ -848,8 +867,8 @@ void Control::stateChanged (Process * p, Process::State os, Process::State ns) {
             NodePtr node = p->mrl (); // p->mrl is weak, needs check
             Mrl * mrl = node ? node->mrl () : 0L;
             if (mrl && !applyBackRequest () &&  // if cause is no jump
-                    mrl->active ()) {        // if cause is eof
-                mrl->endOfFile ();           // set node to finished
+                    mrl->active ()) {           // if cause is eof
+                mrl->message (MsgMediaFinished);// set node to finished
                 if (back_request)            // overlapping audio/video in SMIL
                     g_timeout_add(0, playCurrentSchedule, (void *) this);
             }
@@ -944,6 +963,14 @@ void Control::addRecent (const String &url) {
     }
 }
 
+void Control::setCurrentSelected (NodePtr node) {
+    current_selected = node;
+    if (m_app->fullscreen && VIEWAREA(m_app)->controls)
+        gdk_window_clear_area_e (VIEWAREA(m_app)->controls,
+                110, 0, 410, 60);
+        //gdk_window_invalidate_rect (VIEWAREA(m_app)->controls, NULL, false);
+}
+
 void Control::errorMsg (const String & msg) {
     hildon_banner_show_information (GTK_WIDGET (m_app->window),
             NULL, (const char *) msg);
@@ -973,7 +1000,8 @@ void Control::setPosition (int p) {
         in_progress_update = true;
         if (progress_banner)
             setLoading (100);
-        hildon_seekbar_set_position (HILDON_SEEKBAR (m_app->progress_bar), p);
+        if (p > 0)
+            hildon_seekbar_set_position(HILDON_SEEKBAR(m_app->progress_bar), p);
         in_progress_update = false;
     }
 }
@@ -1026,8 +1054,21 @@ void app_init_control (Application *app) {
     CONTROL(app)->init ();
 }
 
+static Mrl *findFirstPlayable (Node *n) {
+    if (n) {
+        if (n->isPlayable ())
+            return n->mrl ();
+        for (Node *c = n->firstChild().ptr (); c; c = c->nextSibling().ptr ()) {
+            Mrl *mrl = findFirstPlayable (c);
+            if (mrl)
+                return mrl;
+        }
+    }
+    return NULL;
+}
+
 void KMPlayer::jump (Node *n, Control *ctr) {
-    Mrl * mrl = n ? n->mrl () : 0L;
+    Mrl *mrl = findFirstPlayable (n);
     if (mrl && mrl->view_mode == Mrl::SingleMode) {
         if (!mrl->isPlayable ())
             return;
@@ -1138,10 +1179,17 @@ void newFile (Application * app) {
     CONTROL(app)->cur_url = String ();
 }
 
-void cb_mainview_deleted (GtkObject *, Application * app) {
+gboolean cb_mainview_delete (GtkObject *, GdkEvent *, Application *app) {
     CONTROL(app)->deinit ();
     gtk_main_quit ();
-    debugLog () << "destoyed" << endl;
+    debugLog () << "delete" << endl;
+    return false;
+}
+
+void cb_mainview_close (GtkObject *, Application *app) {
+    CONTROL(app)->deinit ();
+    gtk_main_quit ();
+    debugLog () << "close" << endl;
 }
 
 gboolean cb_keyPressed (GtkWidget *, GdkEventKey * e, Application *app) {
@@ -1176,6 +1224,18 @@ gboolean cb_keyPressed (GtkWidget *, GdkEventKey * e, Application *app) {
         case GDK_Return: // HILDON_HARDKEY_SELECT
             rowSelected (app);
             break;
+        case GDK_Up:
+            if (app->fullscreen)
+                previousSelected (app);
+            break;
+        case GDK_Down:
+            if (app->fullscreen)
+                nextSelected (app);
+            break;
+        case GDK_Escape:
+            if (app->fullscreen)
+                VIEWAREA(app)->controlsVisible (true);
+            break;
         default:
             KMPlayer::debugLog () << "Keypress " << e->keyval << KMPlayer::endl;
     }
@@ -1203,7 +1263,7 @@ void toggleMediaServer (GtkCheckMenuItem *radio, Application *app) {
 #endif
     else
         p = CONTROL(app)->mplayer;
-    CONTROL(app)->setProcess (p);
+    CONTROL(app)->setProcess (p->name ());
 #ifdef __ARMEL__
     if (p != CONTROL(app)->npp)
 #endif
