@@ -80,6 +80,7 @@ public:
     ViewArea *viewArea () const;
     unsigned long videoOutput () const;
     void addRecent (const String &url);
+    void setCurrentSelected (NodePtr node);
     /**
      * PlayListNotify implementation
      */
@@ -150,6 +151,8 @@ private:
 void updateTree (Application *app, short id) MAEMO_NO_EXPORT;
 void updatePlayTree (Application *app) MAEMO_NO_EXPORT;
 void rowSelected (Application *app) MAEMO_NO_EXPORT;
+void nextSelected (Application *app) MAEMO_NO_EXPORT;
+void previousSelected (Application *app) MAEMO_NO_EXPORT;
 short addTree (Application *a, KMPlayer::NodePtr doc, GdkPixbuf *ico) MAEMO_NO_EXPORT;
 void jump (KMPlayer::Node *n, Control *ctr) MAEMO_NO_EXPORT;
 
