@@ -452,11 +452,11 @@ void cb_selection_changed (GtkTreeSelection *selection, Application * app) {
             if (!node->isPlayable ()) // lazy playlist/recents loading
                 updateTree (app, pld->id);
         } else {
-            CONTROL(app)->current_selected = node;
             app_set_info_content (app, node
                     ? (pld->edit_mode ? node->innerXML () : node->innerText ())
                     : "", pld->edit_mode);
         }
+        CONTROL(app)->setCurrentSelected (node);
         if (pld->id > 0) //FIXME, hack to become less jumpy when changing trees
             pld->current = node;
     }
@@ -491,6 +491,64 @@ void KMPlayer::rowSelected (Application * app) {
         jump (node.ptr (), CONTROL(app));
 }
 
+void KMPlayer::nextSelected (Application *app) {
+    GtkTreeIter iter;
+    GtkTreeIter next;
+    GtkTreeView *view = GTK_TREE_VIEW (app->playlist_view);
+    GtkTreeSelection *selection = gtk_tree_view_get_selection (view);
+    GtkTreeModel *model = gtk_tree_view_get_model (view);
+
+    if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+        if (gtk_tree_model_iter_has_child (model, &iter)) {
+            GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
+            if (!gtk_tree_view_row_expanded (view, path))
+                gtk_tree_view_expand_row (view, path, FALSE);
+            gtk_tree_path_free (path);
+            if (gtk_tree_model_iter_children (model, &next, &iter))
+                gtk_tree_selection_select_iter (selection, &next);
+        } else if (gtk_tree_model_iter_next (model, &iter)) {
+            gtk_tree_selection_select_iter (selection, &iter);
+        } else {
+            while (gtk_tree_model_iter_parent (model, &next, &iter)) {
+                if (gtk_tree_model_iter_next (model, &next)) {
+                    gtk_tree_selection_select_iter (selection, &next);
+                    break;
+                }
+                iter = next;
+            }
+        }
+    }
+}
+
+void KMPlayer::previousSelected (Application *app) {
+    GtkTreeIter iter;
+    GtkTreeView *view = GTK_TREE_VIEW (app->playlist_view);
+    GtkTreeSelection *selection = gtk_tree_view_get_selection (view);
+    GtkTreeModel *model = gtk_tree_view_get_model (view);
+
+    if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+        GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
+        if (gtk_tree_path_prev (path))
+            gtk_tree_selection_select_path (selection, path);
+        else if (gtk_tree_path_up (path))
+            gtk_tree_selection_select_path (selection, path);
+        gtk_tree_path_free (path);
+    }
+}
+
+GValue *playlist_selected (Application *app, GValue *value) {
+    GtkTreeIter iter;
+    GtkTreeView *view = GTK_TREE_VIEW (app->playlist_view);
+    GtkTreeSelection *selection = gtk_tree_view_get_selection (view);
+    GtkTreeModel *model = gtk_tree_view_get_model (view);
+
+    if (gtk_tree_selection_get_selected (selection, NULL, &iter)) {
+        gtk_tree_model_get_value (model, &iter, COL_TEXT, value);
+        return value;
+    }
+    return NULL;
+}
+
 void cb_treeCellDataFunc (GtkTreeViewColumn *tree_column, GtkCellRenderer *cell,
         GtkTreeModel *tree_model, GtkTreeIter *iter, gpointer data) {
     GtkTreeIter parent;
