@@ -27,6 +27,7 @@
 #include "kmplayerplaylist.h"
 
 typedef struct DBusPendingCall DBusPendingCall;
+typedef struct _Application Application;
 
 namespace KMPlayer {
 
@@ -271,6 +272,30 @@ private:
 
 #endif //__ARMEL__
 
+/*
+ * Download backend
+ */
+class Downloader : public Process, public AsyncJobInterface {
+public:
+    Downloader (ProcessNotify * ctrl, Application *app);
+    ~Downloader ();
+
+    virtual bool play (NodePtr mrl);
+    virtual bool stop ();
+
+    String saved_file;
+
+private:
+    virtual void jobData (Job *job, ByteArray & data);
+    virtual void jobResult (Job *job);
+
+    Job *job;
+    Application *application;
+    unsigned long size;
+    unsigned long bytes;
+    int fd;
+};
+
 } // namespace
 
 #endif //_KMPLAYERPROCESS_H_
