@@ -31,6 +31,7 @@
 #include "kmplayer_smil.h"
 #include "kmplayer.h"
 #include "viewarea.h"
+#include "kmplayercontrol.h"
 #include "kmplayer_rp.h"
 #include "actor.h"
 
@@ -304,6 +305,7 @@ public:
     void visit (SMIL::ImageMediaType *);
     void visit (SMIL::TextMediaType *);
     void visit (SMIL::Brush *);
+    void visit (SMIL::SmilText *);
     void visit (SMIL::RefMediaType *);
     void visit (SMIL::AVMediaType *);
     void visit (RP::Imfl *);
@@ -935,6 +937,58 @@ KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::Brush * brush) {
     }
 }
 
+KDE_NO_EXPORT void CairoPaintVisitor::visit (SMIL::SmilText *txt) {
+    Surface *s = (Surface *) txt->message (MsgQueryRoleDisplay);
+    if (!s)
+        return;
+
+    SRect rect = s->bounds;
+    Single x = rect.x (), y = rect.y(), w = rect.width(), h = rect.height();
+    matrix.getXYWH (x, y, w, h);
+
+    if (!s->surface) {
+
+        s->surface = cairo_surface_create_similar (cairo_surface,
+                CAIRO_CONTENT_COLOR_ALPHA, (int) w, (int) h);
+        cairo_t *cr_txt = cairo_create (s->surface);
+
+        CAIRO_SET_SOURCE_RGB (cr_txt, 0);
+        PangoLayout *layout = pango_cairo_create_layout (cr_txt);
+        pango_layout_set_width (layout, 1.0 * w * PANGO_SCALE);
+        pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
+
+        pango_layout_set_markup (layout, (const char *) txt->richText (), -1);
+
+        PangoFontDescription *desc = pango_font_description_new ();
+        pango_font_description_set_family (desc, "Sans");
+        pango_font_description_set_absolute_size (desc,
+                PANGO_SCALE * (w * 14 / rect.width ()));
+        pango_layout_set_font_description (layout, desc);
+        pango_font_description_free (desc);
+
+        pango_cairo_show_layout (cr_txt, layout);
+        g_object_unref (layout);
+        cairo_destroy (cr_txt);
+    }
+    IRect clip_rect = clip.intersect (IRect (x, y, w, h));
+    if (!clip_rect.isEmpty ()) {
+        cairo_save (cr);
+        cairo_matrix_init_translate (&cur_mat, (int)-x, (int)-y);
+        cairo_pattern_t *pat = cairo_pattern_create_for_surface (s->surface);
+        cairo_pattern_set_extend (pat, CAIRO_EXTEND_NONE);
+        cairo_pattern_set_matrix (pat, &cur_mat);
+        cairo_pattern_set_filter (pat, CAIRO_FILTER_FAST);
+        cairo_set_source (cr, pat);
+        cairo_rectangle(cr, clip_rect.x, clip_rect.y, clip_rect.w, clip_rect.h);
+        cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
+        cairo_fill (cr);
+        cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+        cairo_pattern_destroy (pat);
+        cairo_restore (cr);
+    }
+    s->dirty = false;
+}
+
 KDE_NO_EXPORT void CairoPaintVisitor::visit (RP::Imfl * imfl) {
     if (imfl->surface ()) {
         cairo_save (cr);
@@ -1464,7 +1518,8 @@ public:
         backing_store = XCreatePixmap (d, wid, w, h, gdk_drawable_get_depth (
                     GDK_DRAWABLE (m_view_area->viewWidget ()->window)));
         return cairo_xlib_surface_create (d, backing_store,
-                DefaultVisual (d, DefaultScreen (d)), w, h);
+                gdk_x11_visual_get_xvisual (gtk_widget_get_visual (
+                        m_view_area->viewWidget ())), w, h);
         /*return cairo_xlib_surface_create_with_xrender_format (
             QX11Info::display (),
             id,
@@ -1508,6 +1563,8 @@ public:
 
 KDE_NO_CDTOR_EXPORT ViewArea::ViewArea ()
  : d (new ViewAreaPrivate (this)),
+   controls (NULL),
+   controls_timer (0),
    m_updaters (NULL),
    aspect (0.0),
    widget (0L),
@@ -1778,14 +1835,16 @@ KDE_NO_EXPORT bool ViewArea::timeoutPaintEvent () {
 }
 
 KDE_NO_EXPORT int ViewArea::width () const {
-    int w, h;
-    gdk_drawable_get_size (widget->window, &w, &h);
+    int w = 0, h;
+    if (widget)
+        gdk_drawable_get_size (widget->window, &w, &h);
     return w;
 }
 
 KDE_NO_EXPORT int ViewArea::height () const {
-    int w, h;
-    gdk_drawable_get_size (widget->window, &w, &h);
+    int w, h = 0;
+    if (widget)
+        gdk_drawable_get_size (widget->window, &w, &h);
     return h;
 }
 
@@ -1831,18 +1890,24 @@ static bool isChildWindow (Display *dpy, Window p, Window c) {
     return found;
 }
 
-GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, gpointer d) {
+static
+GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, Application *a) {
     XEvent *event = (XEvent *) xe;
+    ViewArea *view_area = VIEWAREA(a);
+
+    if (view_area->controls &&
+            event->xany.window == GDK_DRAWABLE_XID (view_area->controls))
+        return GDK_FILTER_CONTINUE;
+
     switch (event->xany.type) {
         case KeyPress:
         case KeyRelease: {
-            Window v = GDK_DRAWABLE_XID (GDK_DRAWABLE (
-                        ((ViewArea *)d)->videoWindow ()));
+            Window v = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow()));
             if (event->xany.window == v ||
                     isChildWindow(event->xany.display, v, event->xkey.window)) {
                 XKeyEvent keyevent = event->xkey;
                 keyevent.window =
-                    GDK_DRAWABLE_XID (GDK_DRAWABLE (((ViewArea *)d)->viewWidget ()->window));
+                    GDK_DRAWABLE_XID (GDK_DRAWABLE (view_area->viewWidget ()->window));
                 XSendEvent (event->xany.display, keyevent.window, true,
                         KeyPressMask, (XEvent *) &keyevent);
                 return GDK_FILTER_REMOVE;
@@ -1850,8 +1915,7 @@ GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, gpointer d) {
             break;
         }
         case MapNotify: {
-            Window v = GDK_DRAWABLE_XID (GDK_DRAWABLE (
-                        ((ViewArea *)d)->videoWindow ()));
+            Window v = GDK_DRAWABLE_XID(GDK_DRAWABLE(view_area->videoWindow()));
             if (!event->xmap.override_redirect &&
                  (event->xany.window == v ||
                   isChildWindow (event->xany.display, v, event->xmap.window))) {
@@ -1859,16 +1923,98 @@ GdkFilterReturn videoWidgetFilter (GdkXEvent *xe, GdkEvent *e, gpointer d) {
                         event->xmap.window,
                         KeyPressMask|ExposureMask|SubstructureNotifyMask);
             }
+            break;
+        }
+        case ButtonPress:
+            if (a->fullscreen &&
+                    event->xany.window == GDK_DRAWABLE_XID (GDK_DRAWABLE (view_area->viewWidget ()->window)) &&
+                    event->xbutton.x > 720 && event->xbutton.y < 60)
+                view_area->controlsVisible (true);
+            break;
+    }
+    return GDK_FILTER_CONTINUE;
+}
+
+static
+GdkFilterReturn controlsFilter (GdkXEvent *xe, GdkEvent *e, Application *app) {
+    XEvent *event = (XEvent *) xe;
+    ViewArea *view_area = VIEWAREA(app);
+
+    if (event->xany.window != GDK_DRAWABLE_XID (view_area->controls))
+        return GDK_FILTER_CONTINUE;
+
+    switch (event->xany.type) {
+    case ButtonPress:
+        if (app->fullscreen) {
+            int x, w = -1;
+            if (event->xbutton.x < 100) {
+                previousSelected (app);
+                x = 0;
+                w = 100;
+            } else if (event->xbutton.x > 540 && event->xbutton.x < 610) {
+                nextSelected (app);
+                x = 540;
+                w = 70;
+            } else if (event->xbutton.x > 610 && event->xbutton.x < 720) {
+                rowSelected (app);
+                x = 610;
+                w = 110;
+            } else if (event->xbutton.x > 720) {
+                view_area->controlsVisible (false);
+                x = 720;
+                w = 80;
+            }
+            if (w > 0) {
+                gdk_window_clear_area_e (VIEWAREA(app)->controls, x, 0, w, 60);
+                if (x < 720)
+                    view_area->controlsVisible (true);
+            }
+        }
+        break;
+    case Expose:
+        if (!event->xexpose.count && event->xany.window &&
+                event->xany.window == GDK_DRAWABLE_XID (view_area->controls)) {
+            cairo_t *cr = gdk_cairo_create (view_area->controls);
+            cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+            cairo_set_source_rgb (cr, 32.0 / 255, 32.0 / 255, 1);
+            cairo_rectangle (cr, 0, 0, 610, 60);
+            cairo_fill (cr);
+            cairo_set_source_rgb (cr, 32.0 / 255, 1, 48.0 / 255);
+            cairo_rectangle (cr, 610, 0, 105, 60);
+            cairo_fill (cr);
+            cairo_set_source_rgb (cr, 1, 32.0 / 255, 48.0 / 255);
+            cairo_rectangle (cr, 720, 0, 80, 60);
+            cairo_fill (cr);
+            GValue value = { 0 , };
+            if (playlist_selected (app, &value)) {
+                const gchar *val = g_value_get_string (&value);
+                cairo_set_source_rgb (cr, 0.8, 0.8, 0);
+                cairo_move_to (cr, 130, 10);
+                PangoLayout *layout = pango_cairo_create_layout (cr);
+                pango_layout_set_width (layout, 380 * PANGO_SCALE);
+                pango_layout_set_text (layout, val, -1);
+                PangoFontDescription *desc = pango_font_description_new ();
+                pango_font_description_set_family (desc, "Sans");
+                pango_font_description_set_absolute_size (desc,
+                        PANGO_SCALE * 20);
+                pango_layout_set_font_description (layout, desc);
+                pango_font_description_free (desc);
+                pango_cairo_show_layout (cr, layout);
+                g_object_unref (layout);
+                g_value_unset (&value);
+            }
+            cairo_destroy (cr);
+            return GDK_FILTER_REMOVE;
         }
     }
     return GDK_FILTER_CONTINUE;
 }
 
-KDE_NO_EXPORT void ViewArea::setEventFiltering (bool on) {
+KDE_NO_EXPORT void ViewArea::setEventFiltering (Application *app, bool on) {
     if (on && !m_event_filter_on)
-        gdk_window_add_filter (NULL, videoWidgetFilter, this);
+        gdk_window_add_filter (NULL, (GdkFilterFunc) videoWidgetFilter, app);
     else if (!on && m_event_filter_on)
-        gdk_window_remove_filter (NULL, videoWidgetFilter, this);
+        gdk_window_remove_filter (NULL, (GdkFilterFunc) videoWidgetFilter, app);
     m_event_filter_on = on;
 }
 
@@ -1907,15 +2053,33 @@ KDE_NO_EXPORT void ViewArea::setOutputWidget (Application *app, GtkWidget * w) {
     //resizeEvent ();
 }
 
+gboolean cb_controlsTimeout (ViewArea *view_area) {
+    view_area->controls_timer = 0;
+    view_area->controlsVisible (false);
+    return false;
+}
+
+void ViewArea::controlsVisible (bool show) {
+    if (controls_timer)
+        g_source_remove (controls_timer);
+    if (show) {
+        if (!controls_timer)
+            gdk_window_move (controls, 0, 420);
+        controls_timer = g_timeout_add (10000, (GSourceFunc) cb_controlsTimeout, this);
+    } else {
+        if (controls)
+            gdk_window_move (controls, 0, 490);
+        controls_timer = 0;
+    }
+}
+
 //------------------------------------------------------------------------------
 
 void cb_view_area_button_press (GtkWidget *w, GdkEventButton *e, Application * app) {
-        //debugLog () << "button press " << event->x << "," << event->y << endl;
     VIEWAREA(app)->mousePressEvent (int (e->x), int (e->y));
 }
 
 void cb_view_area_expose (GtkWidget *w, GdkEventExpose *e, Application *app) {
-    debugLog () << "cb_view_area_expose " << e->area.x << "," << e->area.y << " " << e->area.width << "x" << e->area.height << endl;
     VIEWAREA(app)->paintEvent (
             IRect (e->area.x, e->area.y, e->area.width, e->area.height));
 }
@@ -1925,22 +2089,63 @@ void cb_view_area_realized (GtkWidget *widget, Application * app) {
     if (widget == app->view_area_fullscreen) {
         Color c (0x101010);
         gdk_window_set_background (widget->window, (GdkColor *)(GColor)c);
-    } else
+    } else {
         VIEWAREA(app)->setOutputWidget (app, widget);
+    }
 }
 
+#include "controls.xbm"
+
 gboolean cbViewAreaFullscreenMapped (GtkWidget *w, GdkEvent *, Application *a) {
+    ViewArea *view_area = VIEWAREA(a);
     gtk_window_fullscreen (GTK_WINDOW (a->window_fullscreen));
-    VIEWAREA(a)->setOutputWidget (a, a->view_area_fullscreen);
+    view_area->setOutputWidget (a, a->view_area_fullscreen);
+    if (!view_area->controls) {
+        GdkWindowAttr attributes;
+        attributes.window_type = GDK_WINDOW_CHILD;
+        attributes.x = 0;
+        attributes.y = 0;
+        attributes.width = 800;
+        attributes.height = 60;
+        attributes.wclass = GDK_INPUT_OUTPUT;
+        attributes.visual = gtk_widget_get_visual (w);
+        attributes.colormap = gtk_widget_get_colormap (w);
+        attributes.event_mask = gtk_widget_get_events (w) |
+            GDK_SUBSTRUCTURE_MASK | GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK;
+        attributes.event_mask &= ~GDK_KEY_PRESS_MASK;
+        //    ~(GDK_KEY_PRESS_MASK | GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);
+        gint mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+        view_area->controls = gdk_window_new (a->view_area_fullscreen->window, &attributes, mask);
+        gdk_window_add_filter (view_area->controls, (GdkFilterFunc) controlsFilter, a);
+        GdkBitmap *bitmap = gdk_bitmap_create_from_data (NULL,
+                (const gchar *) controls_bits, controls_width, controls_height);
+        gdk_window_shape_combine_mask (view_area->controls, bitmap, 0, 0);
+        gdk_window_show (view_area->controls);
+    } else {
+        gdk_window_raise (view_area->controls);
+    }
+    view_area->controlsVisible (false);
     cb_view_area_configure (w, 0L, a);
     return false;
 }
 
 void cbViewAreaFullscreenUnmapped (GtkWidget *w, Application *a) {
-    VIEWAREA(a)->setOutputWidget (a, a->view_area);
+    ViewArea *view_area = VIEWAREA(a);
+    view_area->setOutputWidget (a, a->view_area);
+    view_area->controlsVisible (false);
     cb_view_area_configure (w, 0L, a);
 }
 
+void cb_fullscreen_destoyed (GtkObject *o, Application *app) {
+    if (app->fullscreen) {
+        ViewArea *view_area = VIEWAREA(app);
+        app->fullscreen = false;
+        app->window_fullscreen = NULL;
+        view_area->controls = NULL;
+        view_area->controlsVisible (false);
+    }
+}
+
 gboolean cb_scheduledPaint (void * vp) {
     return static_cast <ViewArea *> (vp)->timeoutPaintEvent ();
 }
